@@ -112,28 +112,28 @@ s_attribute_types = {}
 # Comprehension-optional range (0x8000-0xFFFF)
 
 # [RFC-3489]
-MappedAddress = AttributeType(0x0001, name='Mapped Address')
-ResponseAddress = AttributeType(0x0002, name='Response Address')
-ChangeRequest = AttributeType(0x0003, name='Change Request')
-SourceAddress = AttributeType(0x0004, name='Source Address')
-ChangedAddress = AttributeType(0x0005, name='Changed Address')
-Username = AttributeType(0x0006, name='Username')
-Password = AttributeType(0x0007, name='Password')
-MessageIntegrity = AttributeType(0x0008, name='Message Integrity')
-ErrorCode = AttributeType(0x0009, name='Error Code')
-UnknownAttribute = AttributeType(0x000A, name='Unknown Attribute')
-ReflectedFrom = AttributeType(0x000B, name='Reflected From')
+MappedAddress = AttributeType(0x0001, name='MAPPED-ADDRESS')
+ResponseAddress = AttributeType(0x0002, name='RESPONSE-ADDRESS')
+ChangeRequest = AttributeType(0x0003, name='CHANGE-REQUEST')
+SourceAddress = AttributeType(0x0004, name='SOURCE-ADDRESS')
+ChangedAddress = AttributeType(0x0005, name='CHANGED-ADDRESS')
+Username = AttributeType(0x0006, name='USERNAME')
+Password = AttributeType(0x0007, name='PASSWORD')
+MessageIntegrity = AttributeType(0x0008, name='MESSAGE-INTEGRITY')
+ErrorCode = AttributeType(0x0009, name='ERROR-CODE')
+UnknownAttributes = AttributeType(0x000A, name='UNKNOWN-ATTRIBUTES')
+ReflectedFrom = AttributeType(0x000B, name='REFLECTED-FROM')
 
 # [RFC-5389]
-Realm = AttributeType(0x0014, name='Realm')
-Nonce = AttributeType(0x0015, name='Nonce')
-XorMappedAddress = AttributeType(0x0020, name='Xor Mapped Address')
+Realm = AttributeType(0x0014, name='REALM')
+Nonce = AttributeType(0x0015, name='NONCE')
+XorMappedAddress = AttributeType(0x0020, name='XOR-MAPPED-ADDRESS(0020)')
 
-XorMappedAddress2 = AttributeType(0x8020, name='(Xor?) Mapped Address')
-XorOnly = AttributeType(0x8021, name='XOR Only')
-Software = AttributeType(0x8022, name='Software')
-AlternateServer = AttributeType(0x8023, name='Alternate Server')
-Fingerprint = AttributeType(0x8028, name='Fingerprint')
+XorMappedAddress2 = AttributeType(0x8020, name='XOR-MAPPED-ADDRESS(8020)')
+XorOnly = AttributeType(0x8021, name='XOR-ONLY')
+Software = AttributeType(0x8022, name='SOFTWARE')
+AlternateServer = AttributeType(0x8023, name='ALTERNATE-SERVER')
+Fingerprint = AttributeType(0x8028, name='FINGERPRINT')
 
 
 class Attribute(TLV):
@@ -240,43 +240,52 @@ class MappedAddressValue(AttributeValue):
         return self.__ip
 
     @classmethod
-    def parse(cls, data: bytes, length: int):
-        if data[0] != 0:
-            return None
-        family = bytes_to_int(data[1:2])
-        port = bytes_to_int(data[2:4])
-        address = data[4:]
-        # check address family
-        if family == cls.family_ipv4:
-            assert length == 8, 'IPv4 data error: %d' % length
-            # IPv4
-            ip = '.'.join([
-                str(bytes_to_int(address[0:1])),
-                str(bytes_to_int(address[1:2])),
-                str(bytes_to_int(address[2:3])),
-                str(bytes_to_int(address[3:4])),
-            ])
-        elif family == cls.family_ipv6:
-            assert length == 20, 'IPv6 data error: %d' % length
-            # TODO: IPv6
-            assert False, 'implement me!'
-        else:
-            raise ValueError('unknown address family: %d' % family)
-        return cls(data=data, ip=ip, port=port, family=family)
-
-    @classmethod
-    def new(cls, ip: str, port: int, family: int = family_ipv4):
+    def ip_to_bytes(cls, ip: str, family: int) -> bytes:
         if family == cls.family_ipv4:
             # IPv4
             array = ip.split('.')
             assert len(array) == 4, 'IP address error: %s' % ip
-            address = bytes([int(x) for x in array])
+            return bytes([int(x) for x in array])
             pass
         elif family == cls.family_ipv6:
             # TODO: IPv6
             assert False, 'implement me!'
         else:
             raise ValueError('unknown address family: %d' % family)
+
+    @classmethod
+    def bytes_to_ip(cls, address: bytes, family: int) -> str:
+        # check address family
+        if family == cls.family_ipv4:
+            assert len(address) == 4, 'IPv4 data error: %s' % address
+            # IPv4
+            return '.'.join([
+                str(bytes_to_int(address[0:1])),
+                str(bytes_to_int(address[1:2])),
+                str(bytes_to_int(address[2:3])),
+                str(bytes_to_int(address[3:4])),
+            ])
+        elif family == cls.family_ipv6:
+            assert len(address) == 16, 'IPv6 data error: %s' % address
+            # TODO: IPv6
+            assert False, 'implement me!'
+        else:
+            raise ValueError('unknown address family: %d' % family)
+
+    @classmethod
+    def parse(cls, data: bytes, length: int):
+        if data[0] != 0:
+            return None
+        family = bytes_to_int(data[1:2])
+        port = bytes_to_int(data[2:4])
+        ip = cls.bytes_to_ip(address=data[4:], family=family)
+        return cls(data=data, ip=ip, port=port, family=family)
+
+    @classmethod
+    def new(cls, ip: str, port: int, family: int=None):
+        if family is None:
+            family = cls.family_ipv4
+        address = cls.ip_to_bytes(ip=ip, family=family)
         data = b'\0' + uint8_to_bytes(family) + uint16_to_bytes(port) + address
         return cls(data=data, ip=ip, port=port, family=family)
 
@@ -359,6 +368,16 @@ class XorMappedAddressValue(MappedAddressValue):
             f_pos += 1
         return array
 
+    @classmethod
+    def new(cls, ip: str, port: int, family: int=None, factor: bytes=None):
+        if family is None:
+            family = cls.family_ipv4
+        address = cls.ip_to_bytes(ip=ip, family=family)
+        data = b'\0' + uint8_to_bytes(family) + uint16_to_bytes(port) + address
+        assert factor is not None, 'missing factor'
+        data = cls.xor(data=data, factor=factor)
+        return cls(data=data, ip=ip, port=port, family=family)
+
 
 class XorMappedAddressValue2(MappedAddressValue):
     """ https://tools.ietf.org/id/draft-ietf-behave-rfc3489bis-02.txt
@@ -418,6 +437,16 @@ class XorMappedAddressValue2(MappedAddressValue):
             f_pos += 1
         return array
 
+    @classmethod
+    def new(cls, ip: str, port: int, family: int=None, factor: bytes=None):
+        if family is None:
+            family = cls.family_ipv4
+        address = cls.ip_to_bytes(ip=ip, family=family)
+        data = b'\0' + uint8_to_bytes(family) + uint16_to_bytes(port) + address
+        assert factor is not None, 'missing factor'
+        data = cls.xor(data=data, factor=factor)
+        return cls(data=data, ip=ip, port=port, family=family)
+
 
 class ResponseAddressValue(MappedAddressValue):
     """
@@ -542,6 +571,17 @@ class SoftwareValue(AttributeValue):
         desc = data.rstrip(b'\0').decode('utf-8')
         return cls(data=data, description=desc)
 
+    @classmethod
+    def new(cls, description: str):
+        data = description.encode('utf-8')
+        # padding with '\0'
+        length = len(data)
+        tail = length & 3
+        while tail < 4:
+            data += b'\0'
+            tail += 1
+        return cls(data=data, description=description)
+
 
 #
 #  Register attribute parsers
